using System.Collections;
using System.Collections.Generic;
using TMPro;
using UnityEngine;
using UnityEngine.UI;

namespace Mahjong
{
    public class GameManager: MonoBehaviour
    {
        public static GameManager Instance;
        public int MaxProblem;
        public HandGenerator HandGenerator;
        public HandsPanel HandsPanel;
        public HandSpriteTable Table;
        public Evaluator Evaluator;
        public EvaluateData EvaluateData;
        public HandData HandData;
        public GameData GameData;
        public TMP_Text Prompt;
        public int[] UsedHands;

        [SerializeField] private Button generateHandButton;
        [SerializeField] private Button doDebugButton;
        [SerializeField] private Button startReviewButton, nextReviewButton, startNewGameButton, endGameButton;
        [SerializeField] private GameDataIndicator gameDataIndicator;
        [SerializeField] private InputButton inputButtonPrefab;
        [SerializeField] private Transform inputField_Normal, inputField_Yukman;
        [SerializeField] private Statistics statistics;
        [SerializeField] private bool isDebug;
        private bool updateTime, endGame;


        private void Awake()
        {
            if (!Instance)
            {
                Instance = this;
                DontDestroyOnLoad(this.gameObject);
            }
            else
                Destroy(this.gameObject);
        }

        private void Start()
        {
            generateHandButton.onClick.AddListener(GenerateNewGame);
            doDebugButton.onClick.AddListener(DoDebug);
            startReviewButton.onClick.AddListener(StartReview);
            nextReviewButton.onClick.AddListener(NextReview);
            startNewGameButton.onClick.AddListener(GenerateNewGameSet);
            endGameButton.onClick.AddListener(QuitGame);
            GenerateNewGameSet();
        }

        private void Update()
        {
            if (endGame)
                return;

            if(updateTime)
                statistics.UpdateTime();
        }

        private void GenerateInputButton()
        {
            for(int i=1; i<13; i++)
            {
                if(i<=6)
                    Instantiate(inputButtonPrefab, inputField_Yukman).Init(i, true);
                Instantiate(inputButtonPrefab, inputField_Normal).Init(i, false);
            }
        }

        private void DestroyInputButton()
        {
            for (int i = 1; i < 13; i++)
            {
                if (i <= 6)
                    Destroy(inputField_Yukman.GetChild(i-1).gameObject);
                Destroy(inputField_Normal.GetChild(i-1).gameObject);
            }
        }

        public void GenerateNewGameSet()
        {
            startReviewButton.gameObject.SetActive(false);
            nextReviewButton.gameObject.SetActive(false);
            startNewGameButton.gameObject.SetActive(false);
            GenerateInputButton();
            statistics.Reset();
            updateTime = true;
            endGame = false;
            GenerateNewGame();
        }

        public void GenerateNewGame()
        {
            startReviewButton.gameObject.SetActive(false);
            nextReviewButton.gameObject.SetActive(false);
            HandData = HandGenerator.GenerateHand();
            HandsPanel.UpdateHands(HandData);
            GameData = GameData.New();
            gameDataIndicator.UpdateData(GameData);
            EvaluateData = Evaluator.Evaluate(true);
            if (isDebug)
                Prompt.gameObject.SetActive(true);
            else
                Prompt.gameObject.SetActive(false);

            if (EvaluateData.Grade == 0)
                GenerateNewGame();
            else
            {
                statistics.currentProblem += 1;
                statistics.UpdateText();
            }
                
        }

        private void DoDebug()
        {
            HandsPanel.UpdateHands(HandData);
            gameDataIndicator.UpdateData(GameData);
            EvaluateData = Evaluator.Evaluate(true);
        }

        public void CheckAnswer(EvaluateData inputData)
        { 
            if (endGame)
                return;

            statistics.maxAnswer += 1;
            if (inputData.Grade == EvaluateData.Grade && inputData.Is_Yukman == EvaluateData.Is_Yukman)
                statistics.correctAnswer += 1;
            else
                statistics.wrongAnswerList.Add(WrongAnswer.By(HandData,GameData,inputData));
            statistics.UpdateText();
            if (statistics.currentProblem < MaxProblem)
                GenerateNewGame();
            else
                EndGame();
        }

        private void EndGame()
        {
            updateTime = false;
            DestroyInputButton();
            gameDataIndicator.EndGame();
            startNewGameButton.gameObject.SetActive(true);
            HandsPanel.RemoveRemains();
            if (statistics.correctAnswer == statistics.maxAnswer)
            {
                startReviewButton.gameObject.SetActive(false);
                Prompt.text = " ϴ";
            }
            else
            {
                startReviewButton.gameObject.SetActive(true);
                statistics.reviewIndex = 0;
            }

        }
        
        private void StartReview()
        {
            ShowReview();
            startReviewButton.gameObject.SetActive(false);
            nextReviewButton.gameObject.SetActive(true);
            
        }

        private void NextReview()
        {
            if(statistics.reviewIndex < statistics.wrongAnswerList.Count - 1)
            {
                statistics.reviewIndex += 1;
                ShowReview();
            }
            else
            {
                Prompt.text = "  Ȯ߽ϴ";
                nextReviewButton.gameObject.SetActive(false);
            }
        }

        private void ShowReview()
        {
            var count = statistics.reviewIndex;
            this.HandData = statistics.wrongAnswerList[count].hand;
            this.GameData = statistics.wrongAnswerList[count].game;
            var answeredEvaluateData = statistics.wrongAnswerList[count].answer;
            HandsPanel.UpdateHands(HandData);
            gameDataIndicator.UpdateData(GameData);
            EvaluateData = Evaluator.Evaluate(true);
            Prompt.gameObject.SetActive(true);
            string yukman = answeredEvaluateData.Is_Yukman ? "" : "";
            Prompt.text += $"\nԷ° : {answeredEvaluateData.Grade}" + yukman;
        }

        private void QuitGame()
        {
            Application.Quit();
        }
    }
}