using System;
using System.Linq;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

namespace Mahjong
{
    public struct LastHandCandidate
    {
        public HandType type;
        public int from;
        public static LastHandCandidate By(HandType type, int from)
        {
            var result = new LastHandCandidate();
            result.type = type;
            result.from = from;
            return result;
        }
    }

    public class HandGenerator : MonoBehaviour
    {
        public int[] used;
        private HandData handData;
        
        public HandData GenerateHand()
        {
            used = new int[40];
            handData = HandData.New();
            for (int i = 0; i < 4; i++)
                GenerateBody();
            GenerateHead();
            SortStruct();
            PickNewHand();
            return handData;
        }

        private void SortStruct()
        {
            handData.StructList.OrderBy(x => x.HandType);
        }

        private void PickNewHand()
        {
            LastHand lastHand = new LastHand();
            List<LastHandCandidate> lastCandidates = GetLastCandidates();
            int randomIndex = UnityEngine.Random.Range(0, lastCandidates.Count());
            lastHand.HandType = lastCandidates[randomIndex].type;
            lastHand.LastHandFrom = lastCandidates[randomIndex].from;
            int randomInt = UnityEngine.Random.Range(0, 100);
            if (IncludeGangzi() && randomInt < 10)
                lastHand.LastHandType = LastHandType.Youngsang;
            else if (randomInt < 20)
                lastHand.LastHandType = LastHandType.Ron;
            else
                lastHand.LastHandType = LastHandType.Normal;
            handData.LastHand = lastHand;
        }

        private List<LastHandCandidate> GetLastCandidates()
        {
            List<LastHandCandidate> result = new List<LastHandCandidate>();
            for(int i=0; i<handData.StructList.Count(); i++)
            {
                if (handData.StructList[i].StructType == HandStructType.Shunzi)
                {
                    result.Add(LastHandCandidate.By(handData.StructList[i].HandType, i));
                    result.Add(LastHandCandidate.By(handData.StructList[i].HandType + 1, i));
                    result.Add(LastHandCandidate.By(handData.StructList[i].HandType + 2, i));
                }

                else if (handData.StructList[i].StructType == HandStructType.Kezi)
                {
                    result.Add(LastHandCandidate.By(handData.StructList[i].HandType, i));
                    result.Add(LastHandCandidate.By(handData.StructList[i].HandType, i));
                    result.Add(LastHandCandidate.By(handData.StructList[i].HandType, i));
                }

                else if (handData.StructList[i].StructType == HandStructType.Head)
                {
                    result.Add(LastHandCandidate.By(handData.StructList[i].HandType, i));
                    result.Add(LastHandCandidate.By(handData.StructList[i].HandType, i));
                }
            }
            return result;
        }

        private bool IncludeGangzi()
        {
            foreach (var handStruct in handData.StructList)
            {
                if (handStruct.StructType == HandStructType.Gangzi_In || handStruct.StructType == HandStructType.Gangzi_Out)
                    return true;
            }
            return false;
        }

        private void GenerateHead()
        {
            bool succeed = false;
            while(succeed == false)
            {
                HandType type = GetRandomHand();
                int value = (int)type;
                if (used[value] > 2)
                    succeed = false;
                else
                {
                    handData.StructList.Add(HandStruct.By(type, HandStructType.Head));
                    used[value] += 2;
                    succeed = true;
                }
            }
            
        }

        private void GenerateBody()
        {
            bool succeed = false;
            while(succeed == false)
            {
                HandType type = GetRandomHand();
                int value = (int)type;
                if (value < 10)
                    succeed = GenerateTriple(type);
                else
                {
                    int isTriple = UnityEngine.Random.Range(0, 2);
                    if (isTriple == 1)
                        succeed = GenerateTriple(type);
                    else
                        succeed = GenerateStraight(type);
                }
            }            
        }

        private bool GenerateTriple(HandType type)
        {
            int value = (int)type;
            if (used[value] > 1)
                return false;
            else
            {
                int randomInt = UnityEngine.Random.Range(0, 100);
                if (randomInt < 3 && used[value] == 0)
                {
                    handData.StructList.Add(HandStruct.By(type, HandStructType.Gangzi_In));
                    used[value] += 4;
                }
                    
                else if (randomInt < 6 && used[value] == 0)
                {
                    handData.StructList.Add(HandStruct.By(type, HandStructType.Gangzi_Out));
                    used[value] += 4;
                }
                    
                else if (randomInt < 15)
                {
                    handData.StructList.Add(HandStruct.By(type, HandStructType.Pun));
                    used[value] += 3;
                }

                else
                {
                    handData.StructList.Add(HandStruct.By(type, HandStructType.Kezi));
                    used[value] += 3;
                }
                return true;
            }
        }

        private bool GenerateStraight(HandType type)
        {
            int value = (int)type;
            if (value % 10 < 3)
                return GenerateStraight_Up(type);
            else if (value % 10 > 7)
                return GenerateStraight_Down(type);
            else
            {
                if (UnityEngine.Random.Range(0, 2) == 1)
                    return GenerateStraight_Up(type);
                else
                    return GenerateStraight_Down(type);
            }
        }

        private bool GenerateStraight_Up(HandType type)
        {
            int value = (int)type;
            if (used[value] > 3 || used[value + 1] > 3 || used[value + 2] > 3)
                return false;
            else
            {
                int randomInt = UnityEngine.Random.Range(0, 100);
                if (randomInt < 15)
                    handData.StructList.Add(HandStruct.By(type, HandStructType.Chi));
                else
                    handData.StructList.Add(HandStruct.By(type, HandStructType.Shunzi));

                for (int i=0; i<3; i++)
                {
                    used[value + i] += 1;
                }
                return true;
            }
        }

        private bool GenerateStraight_Down(HandType type)
        {
            int value = (int)type;
            if (used[value] > 3 || used[value - 1] > 3 || used[value - 2] > 3)
                return false;
            else
            {
                int randomInt = UnityEngine.Random.Range(0, 100);
                if (randomInt < 15)
                    handData.StructList.Add(HandStruct.By(type - 2, HandStructType.Chi));
                else
                    handData.StructList.Add(HandStruct.By(type - 2, HandStructType.Shunzi));

                for (int i = 0; i < 3; i++)
                {
                    used[value - i] += 1;
                }
                return true;
            }
        }


        public static HandType GetRandomHand()
        {
            Array values = Enum.GetValues(typeof(HandType));
            return (HandType) values.GetValue(UnityEngine.Random.Range(1, values.Length - 1));
        }
    }
}
